// Constants
// Millisecond conversions
Date.prototype.ONE_SECOND = 1000;
Date.prototype.ONE_MINUTE = Date.prototype.ONE_SECOND * 60;
Date.prototype.ONE_HOUR   = Date.prototype.ONE_MINUTE * 60;
Date.prototype.ONE_DAY    = Date.prototype.ONE_HOUR * 24;
// Max values
Date.prototype.MAX_HOURS  = 23;
Date.prototype.MAX_MINS   = 59;
Date.prototype.MAX_SECS   = 59;
// Time conversion factors
Date.prototype.SECONDS_PER_MINUTE = 60;
Date.prototype.SECONDS_PER_HOUR   = Date.prototype.SECONDS_PER_MINUTE * 60;
Date.prototype.SECONDS_PER_DAY    = Date.prototype.SECONDS_PER_HOUR * 24;
// Days of the week
Date.prototype.SUNDAY     = 0;
Date.prototype.MONDAY     = 1;
Date.prototype.TUESDAY    = 2;
Date.prototype.WEDNESDAY  = 3;
Date.prototype.THURSDAY   = 4;
Date.prototype.FRIDAY     = 5;
Date.prototype.SATURDAY   = 6;
// Days short strings
Date.prototype.SUNDAY_SHORT     = 'Sun';
Date.prototype.MONDAY_SHORT     = 'Mon';
Date.prototype.TUESDAY_SHORT    = 'Tues';
Date.prototype.WEDNESDAY_SHORT  = 'Wed';
Date.prototype.THURSDAY_SHORT   = 'Thurs';
Date.prototype.FRIDAY_SHORT     = 'Fri';
Date.prototype.SATURDAY_SHORT   = 'Sat';
// Months of the year
Date.prototype.JANUARY    = 0;
Date.prototype.FEBRUARY   = 1;
Date.prototype.MARCH      = 2;
Date.prototype.APRIL      = 3;
Date.prototype.MAY        = 4;
Date.prototype.JUNE       = 5;
Date.prototype.JULY       = 6;
Date.prototype.AUGUST     = 7;
Date.prototype.SEPTEMBER  = 8;
Date.prototype.OCTOBER    = 9;
Date.prototype.NOVEMBER   = 10;
Date.prototype.DECEMBER   = 11;
// Month short strings
Date.prototype.JANUARY_SHORT    = 'Jan';
Date.prototype.FEBRUARY_SHORT   = 'Feb';
Date.prototype.MARCH_SHORT      = 'Mar';
Date.prototype.APRIL_SHORT      = 'Apr';
Date.prototype.MAY_SHORT        = 'May';
Date.prototype.JUNE_SHORT       = 'Jun';
Date.prototype.JULY_SHORT       = 'Jul';
Date.prototype.AUGUST_SHORT     = 'Aug';
Date.prototype.SEPTEMBER_SHORT  = 'Sept';
Date.prototype.OCTOBER_SHORT    = 'Oct';
Date.prototype.NOVEMBER_SHORT   = 'Nov';
Date.prototype.DECEMBER_SHORT   = 'Dec';

Date.prototype.startOfDay = function() {
    return new Date(this.getFullYear(), this.getMonth(), this.getDate());
  }

Date.prototype.endOfDay = function() {
  var dayEnd = this.startOfDay();
  dayEnd.setSeconds(this.MAX_SECS);
  dayEnd.setMinutes(this.MAX_MINS);
  dayEnd.setHours(this.MAX_HOURS);

  return dayEnd;
}

Date.prototype.beginningOfDayOffsetByDays = function(dateOffset) {
  var result = this.startOfDay();
  result = new Date(result.getTime() + (this.ONE_DAY * dateOffset));
  return result;
}

Date.prototype.endOfDayOffsetByDays = function(dateOffset) {
  var result = this.endOfDay();
  result = new Date(result.getTime() + (this.ONE_DAY * dateOffset));
  return result;
}

Date.prototype.getTweleveHourTimeString = function() {
  var hours = this.getHours(),
      mins = this.getMinutes(),
      period = hours > 11 ? 'pm' : 'am';

  hours = (hours % 12) ;
  if (hours === 0) {
      hours = 12;
  }

  if (mins < 10) {
    mins = '0' + mins;
  }

  return hours + ':' + mins + ' ' + period;
}

Date.prototype.getShortMonthString = function() {
  var month = this.getMonth();
  switch (month) {
    case this.JANUARY:
      return this.JANUARY_SHORT;
    case this.FEBRUARY:
      return this.FEBRUARY_SHORT;
    case this.MARCH:
      return this.MARCH_SHORT;
    case this.APRIL:
      return this.APRIL_SHORT;
    case this.MAY:
      return this.MAY_SHORT;
    case this.JUNE:
      return this.JUNE_SHORT;
    case this.JULY:
      return this.JULY_SHORT;
    case this.AUGUST:
      return this.AUGUST_SHORT;
    case this.SEPTEMBER:
      return this.SEPTEMBER_SHORT;
    case this.OCTOBER:
      return this.OCTOBER_SHORT;
    case this.NOVEMBER:
      return this.NOVEMBER_SHORT;
    case this.DECEMBER:
      return this.DECEMBER_SHORT;
  };
}

Date.prototype.getShortDayString = function (){
  var day = this.getDay();
  switch (day) {
    case this.SUNDAY:
      return this.SUNDAY_SHORT;
    case this.MONDAY:
      return this.MONDAY_SHORT;
    case this.TUESDAY:
      return this.TUESDAY_SHORT;
    case this.WEDNESDAY:
      return this.WEDNESDAY_SHORT;
    case this.THURSDAY:
      return this.THURSDAY_SHORT;
    case this.FRIDAY:
      return this.FRIDAY_SHORT;
    case this.SATURDAY:
      return this.SATURDAY_SHORT;
  };
}
