/**
 * Usage: Call ActivateFloatingHeader with a CSS selector(string) matching
 *        table elements that should have a floating header
 *
 *  $(document).ready(function() {
 *      ActivateFloatingHeaders("table.tableWithFloatingHeader");
 *  });
 */


function _UpdateTableHeadersScroll() {
    $("div.divTableWithFloatingHeader").each(function() {
        var originalHeaderRow = $(".tableFloatingHeaderOriginal", this);
        var floatingHeaderRow = $(".tableFloatingHeader", this);
        var offset = $(this).offset();
        var scrollTop = $(window).scrollTop() + 50; // 50 = navbar
        // check if floating header should be displayed
        if ((scrollTop > offset.top)) {
            floatingHeaderRow.css("visibility", "visible");
        }
        else {
            floatingHeaderRow.css("visibility", "hidden");
        }
    });
}


function _UpdateTableHeadersResize() {
    $("div.divTableWithFloatingHeader").each(function() {
        var originalHeaderRow = $(".tableFloatingHeaderOriginal", this);
        var floatingHeaderRow = $(".tableFloatingHeader", this);

        // Copy cell widths from original header
        $("th", floatingHeaderRow).each(function(index) {
            var cellWidth = $("th", originalHeaderRow).eq(index).css('width');
            $(this).css('width', cellWidth);
        });

        // Copy row width from whole table
        floatingHeaderRow.css("width", Math.max(originalHeaderRow.width(), $(this).width()) + "px");

    });
}


function ActivateFloatingHeaders(selector_str){
    $(selector_str).each(function() {
        $(this).wrap("<div class=\"divTableWithFloatingHeader\" style=\"position:relative\"></div>");

        // use first row as floating header by default
        var floatingHeaderSelector = "tr:first";
        var explicitFloatingHeaderSelector = "tr.floating-header"
        if ($(explicitFloatingHeaderSelector, this).length){
            floatingHeaderSelector = explicitFloatingHeaderSelector;
        }

        var originalHeaderRow = $(floatingHeaderSelector, this).first();
        var clonedHeaderRow = originalHeaderRow.clone()
        originalHeaderRow.before(clonedHeaderRow);

        clonedHeaderRow.addClass("tableFloatingHeader");
        clonedHeaderRow.css("position", "fixed");
        clonedHeaderRow.css("z-index", "1010");
        clonedHeaderRow.css("top", "50px");
        clonedHeaderRow.css("visibility", "hidden");
        clonedHeaderRow.css("background-color", "#f1f0e3");
        clonedHeaderRow.css("border-bottom", "1px solid rgba(0,0,0,0.125)");

        originalHeaderRow.addClass("tableFloatingHeaderOriginal");
    });
    _UpdateTableHeadersResize();
    _UpdateTableHeadersScroll();
    $(window).scroll(_UpdateTableHeadersScroll);
    $(window).resize(_UpdateTableHeadersResize);
}