Handlebars.registerHelper("secondsToTimeWords", secondsToTimeWords);

Handlebars.registerHelper("messageDateTime", function(inputDate) {
    var today = new Date(),
        startToday = today.startOfDay(),
        endToday = today.endOfDay(),
        startYesterday = today.beginningOfDayOffsetByDays(-1),
        endYesterday = today.endOfDayOffsetByDays(-1),
        oneWeekAgo = today.beginningOfDayOffsetByDays(-7),
        endTwoDaysAgo = today.endOfDayOffsetByDays(-2),
        inputTime, dateParts, dayParts, timeParts;

    if (typeof inputDate === 'string') {
        dateParts = inputDate.split(' ');
        dayParts = dateParts[0].split('-');
        timeParts = dateParts[1].split(':');

        inputDate = new Date();
        // Set Date using UTC Values
        inputDate.setUTCFullYear(Number.parseInt(dayParts[0]));
        inputDate.setUTCDate(Number.parseInt(dayParts[2]));
        inputDate.setUTCMonth(Number.parseInt(dayParts[1]) - 1); // Adjust by one as month is 0 index based

        // Set Time using UTC Values
        inputDate.setUTCHours(Number.parseInt(timeParts[0]));
        inputDate.setUTCMinutes(Number.parseInt(timeParts[1]));
        inputDate.setUTCSeconds(0);
        inputDate.setUTCMilliseconds(0);
    }

    inputTime = inputDate.getTweleveHourTimeString();

    if (inputDate > startToday && inputDate < endToday) {
        return 'Today ' + inputTime;
    } else if (inputDate > startYesterday && inputDate < endYesterday) {
        return 'Yesterday' + inputTime;
    } else if (inputDate > oneWeekAgo && inputDate < endTwoDaysAgo) {
        return inputDate.getShortDayString() + ' ' + inputTime;
    } else {
        return inputDate.getShortMonthString() + ' ' + inputDate.getDate() + ' ' + inputTime;
    }
});

Handlebars.registerHelper("messageCaller", function(message) {
    if (message.resident) {
        return message.resident.first_name + ' ' + message.resident.last_name;
    }

    return formatPhone(message.caller_id_number);
});

Handlebars.registerHelper("buildCallPath", function() {
    return 'Not yet implemented';
});

Handlebars.registerHelper("callPathTime", function(timestamp) {
    return timestamp.getTweleveHourTimeString();
});

Handlebars.registerHelper("maintenanceTech", function(message) {
    if (!Handlebars.Utils.isEmpty(message.user)) {
        var user = message.user;
        return user.first_name + ' ' + user.last_name;
    } else {
        return formatPhone(message.claimed_by_phone_number);
    }
    return '&mdash;';
});

Handlebars.registerHelper("messageNotesCount", function(message) {
    return Handlebars.utils.isEmpty(message.notes) ? 0 : 1;
});

Handlebars.registerHelper("messageOnsiteResponse", function(message) {
    if (!Handlebars.utils.isEmpty(message.ticket)) {
        if (!Handlebars.utils.isEmpty(message.ticket.response_time)) {
            return secondsToTimeWords(message.ticket.response_time, false, '&mdash');
        }
    }
    return '&mdash;';
});

Handlebars.registerHelper("messageRecording", function(message) {
    if (message.recording_file_name !== null && message.recording_updated_at != null) {
        var url = message.recording_url,
            content_type = message.recording_content_type;

        return new Handlebars.SafeString(
            '<span class="ss-icon ss-pika ss-play audio-toggle">' + '<audio class="audio-player" src="' + url + '" type="' + content_type + '"></span>'
        );
    } else {
        return new Handlebars.SafeString(
            '<span class="meta-info ss-icon ss-pika ss-phone" rel="popover" data-content="Message recording could not be located, please call into your mailbox to listen."></span>'
        );
    }
});

Handlebars.registerHelper("messageStatus", function() {
    var retreived = null;
    if (message.message_status_id === AppConstants.lookups.message_statuses.abandoned) {
        return new Handlebars.SafeString('<span class="text-danger">Abandoned</span>');
    }

    if (!Handlebars.utils.isEmpty(message.ticket)) {
        if (!Handlebars.utils.isEmpty(message.ticket.work_time)) {
            return new Handlebars.SafeString('<span class="text-success">Resolved</span>');
        } else if (!Handlebars.utils.isEmpty(message.ticket.response_time)) {
            return new Handlebars.SafeString('<span class="text-info">On The Job</span>');
        }
    } else {
        if (!Handlebars.utils.isEmpty(message.call_path)) {
            retreived = firstMatch(message.call_path, function(element) {
                if (element.type === 'message') {
                    return element.message_event_type_id === AppConstants.lookups.message_event_types.message_playback_started;
                };
            });
        };

        if (retreived) {
            return Handlebars.SafeString('<span class="text-warning">Responding</span>');
        } else {
            return Handlebars.SafeString('<span class="text-danger">In Progress</span>');
        }
    }
});

Handlebars.registerHelper("messageTotalResolution", function(message) {
    if (!Handlebars.utils.isEmpty(message.ticket)) {
        if (!Handlebars.utils.isEmpty(message.ticket.checkout_out_time)) {
            var checkoutTime = new Date(message.ticket.checkout_out_time),
                receivedAt = new Date(message.received_at),
                seconds = (checkoutTime - receivedAt) / 1000; // convert from milliseconds to seconds
            return secondsToTimeWords(seconds, false, '&mdash');
        };
    };

    return '&mdash';
});

Handlebars.registerHelper("messageTotalWork", function() {
    if (!Handlebars.utils.isEmpty(message.ticket)) {
        if (!Handlebars.utils.isEmpty(message.ticket.work_time)) {
            return secondsToTimeWords(message.ticket.work_time, false, '&mdash;');
        }
    }
    return '&mdash;';
});

Handlebars.registerHelper("ticketCheckInLocation", function() {
    return 'Not yet implemented';
});

Handlebars.registerHelper("ticketCheckOutLocation", function() {
    return 'Not yet implemented';
});

Handlebars.registerHelper("isProperty", function(options) {
    var regex = new RegExp('property', 'i'),
        urlPathParts = window.location.pathname.split('/'),
        result = firstMatch(urlPathParts, function(element) {
            return regex.test(element);
        });

    if (result) {
        return options.fn(this);
    }
});

// Private Functions
function secondsToTime(inputSeconds) {
    var days, hours, seconds, minutes, seconds,
        hourSeconds, minuteSeconds, remainingSeconds;

    days = Math.floor(inputSeconds / Date.prototype.SECONDS_PER_DAY);

    hourSeconds = inputSeconds % Date.prototype.SECONDS_PER_DAY;
    hours = Math.floor(hourSeconds / Date.prototype.SECONDS_PER_HOUR);

    minuteSeconds = inputSeconds % Date.prototype.SECONDS_PER_HOUR;
    minutes = Math.floor(minuteSeconds / Date.prototype.SECONDS_PER_MINUTE);

    remainingSeconds = inputSeconds % Date.prototype.SECONDS_PER_MINUTE;
    seconds = Math.ceil(remainingSeconds);

    return {
        'seconds': seconds,
        'minutes': minutes,
        'hours': hours,
        'days': days
    };
}

function formatPhone(phone) {
    if (typeof phone !== 'String' || phone.length != 10) {
        return '&mdash;';
    }

    var areaCode = phone.substring(0, 3),
        exchange = phone.substring(3, 6),
        line = phone.substring(6);

    return '(' + areaCode + ')' + exchange + '-' + line;
}

function firstMatch(searchArray, check) {
    var result = false,
        returnValue;

    for (var i = 0; i < searchArray.length; i++) {
        if (check(searchArray[i])) {
            result = searchArray[i];
            break;
        }
    }
    return result;
}

function secondsToTimeWords(inputSeconds, showSeconds, showForZero) {
    var timeComponents,
        resultTimeComponents = [],
        numericSeconds = parseInt(inputSeconds);
    showSeconds = showSeconds || false;
    showForZero = showForZero || null;

    if (Number.isNaN(numericSeconds)) {
        return showForZero;
    };

    if (numericSeconds === 0) {
        if (showForZero !== null) {
            return new Handlebars.SafeString(showForZero);
        } else {
            return '0s';
        };
    };

    timeComponents = secondsToTime(numericSeconds);

    if (timeComponents.seconds > 0) {
        if (!showSeconds) {
            if (timeComponents.minutes > 0) {
                if (timeComponents.seconds >= 30) {
                    timeComponents.minutes += 1;
                };
            };
            timeComponents.seconds = 0;
        };
    };

    if (timeComponents.days > 0) {
        resultTimeComponents.push(timeComponents.days + 'd');
    };

    if (timeComponents.hours > 0) {
        resultTimeComponents.push(timeComponents.hours + 'h');
    };

    if (timeComponents.minutes > 0) {
        resultTimeComponents.push(timeComponents.minutes + 'm');
    };

    if (timeComponents.seconds > 0) {
        resultTimeComponents.push(timeComponents.seconds + 's');
    };

    return resultTimeComponents.join(" ");
};